#!/usr/bin/python

# Copyright (C) 2008, 2010 Red Hat Inc.
# 
# This file is part of systemtap, and is free software.  You can
# redistribute it and/or modify it under the terms of the GNU General
# Public License (GPL); either version 2, or (at your option) any
# later version.

import re
import sys
import os
from config_opts import config_opts

def parse_module_output():
    # Parse the output file, looking for probe points
    pp_re = re.compile(": (-?\d+) (\S+)$")
    f = open(config_opts['probes_result'], 'r')
    pp = dict()
    line = f.readline()
    while line:
        match = pp_re.search(line)
        if match:
            pp[match.group(2)] =  int(match.group(1))
        line = f.readline()
    f.close()

    if len(pp.keys()) == 0:
        print >>sys.stderr, "No data found?"
        return 1

    # Parse the list of probe points.
    f = open(config_opts['probes_current'], 'r')
    passed = open(config_opts['probes_passed'], 'a')
    failed = open(config_opts['probes_failed'], 'a')
    untriggered = open(config_opts['probes_untriggered'], 'a')
    unregistered = open(config_opts['probes_unregistered'], 'a')
    line = f.readline().strip()
    while line:
        if pp.has_key(line):
            if pp[line] > 0:
                passed.write(line + '\n')
            elif pp[line] == 0:
                untriggered.write(line + '\n')
            elif pp[line] == -1:
                unregistered.write(line + '\n')
            else:
                failed.write(line + '\n')
        line = f.readline().strip()
    f.close()
    passed.close()
    failed.close()
    untriggered.close()
    unregistered.close()
    return 0

def main():
    rc = parse_module_output()
    sys.exit(rc)

if __name__ == "__main__":
    main()



#!/usr/bin/python

# Copyright (C) 2008, 2010 Red Hat Inc.
# 
# This file is part of systemtap, and is free software.  You can
# redistribute it and/or modify it under the terms of the GNU General
# Public License (GPL); either version 2, or (at your option) any
# later version.

import sys
import os
import os.path
import time
import select
from config_opts import config_opts

def run_module():
    # Find the current size of /var/log/messages
    logfile = '/var/log/messages'
    print >>sys.stderr, "Getting size of %s..." % logfile

    start_pos = os.path.getsize(logfile)
    l = open(logfile, 'r')
    l.seek(start_pos)

    # Insert the module
    print >>sys.stderr, "Inserting module..."
    rc = os.system("/sbin/insmod %s" % os.path.join(os.getcwd(),
                                                    'kprobe_module.ko'))
    if os.WEXITSTATUS(rc) != 0:
        # This is actually semi-OK, which is why there is no error
        # message here, only a notice.  This might mean that every
        # probe tried cannot be registered.
        print >>sys.stderr, "Notice: insmod failed"
    else:
        # Run the load generate commands (if any).  Note we ignore the
        # return values, since we don't really care if the commands
        # succeed or not.
        if config_opts.has_key('load_cmds'):
            print >>sys.stderr, "Running commands..."
            num_waits = 0

            # Redirect the output to 'run_module.log' by modifying
            # what stdout/stderr points to.
            sys.stdout.flush()
            old_stdout = os.dup(sys.stdout.fileno())
            sys.stderr.flush()
            old_stderr = os.dup(sys.stderr.fileno())
            fd = os.open('run_module.log', os.O_CREAT | os.O_WRONLY)
            os.dup2(fd, sys.stdout.fileno())
            os.dup2(fd, sys.stderr.fileno())

            # Run the commands.
            pid_list = list()
            for cmd in config_opts['load_cmds']:
                pid = os.spawnvp(os.P_NOWAIT, cmd[0], cmd)
                pid_list.append(pid)
            for pid in pid_list:
                (childpid, status) = os.waitpid(pid, 0)

            # Restore old value of stdout/stderr.
            os.close(fd)
            os.dup2(old_stdout, sys.stdout.fileno())
            os.dup2(old_stderr, sys.stderr.fileno())

        # Remove the module
        print >>sys.stderr, "Removing module..."
        rc = os.system("/sbin/rmmod kprobe_module")
        if os.WEXITSTATUS(rc) != 0:
            print >>sys.stderr, "Error: rmmod failed"
            return -1

    # Now we have to wait until everything is flushed to the logfile
    print >>sys.stderr, "Looking for output..."
    rc = 0
    f = open(config_opts['probes_result'], 'w')
    attempts = 0
    while 1:
        # Find the ending size of /var/log/messages
        end_pos = os.path.getsize(logfile)

        if end_pos < start_pos:
            # The log files have been rotated.  Read any leftover data,
            # then reopen file.
            data = l.read()
            if data:
                f.write(data)

                # See if we can find 'kprobe_module unloaded' in the
                # data we just read.
                if data.find('kprobe_module unloaded') != -1:
                    break
        
            l.close()
            l = open(logfile, 'r')
            start_pos = 0
            continue

        # Try to wait until data is available
        while 1:
            try:
                input, output, exc = select.select([l.fileno()], [], [], 60)
                break
            except select.error, err:
                if err[0] != EINTR:
                    raise

        # Get the new stuff logged to /var/log/messages
        data = l.read(end_pos - start_pos + 1)
        if not data:
            # ignore EOF
            time.sleep(2)
            attempts += 1
            if attempts < 30:
                continue
            else:
                print >>sys.stderr, "Error: Couldn't find data"
                rc = -1
                break

        # Write results data
        f.write(data)

        # See if we can find 'kprobe_module unloaded' in the data we
        # just read.
        if data.find('kprobe_module unloaded') == -1:
            start_pos = end_pos
        else:
            break

    l.close()
    f.close()
    return rc

def main():
    rc = run_module()
    sys.exit(rc)

if __name__ == "__main__":
    main()



#!/usr/bin/python

# Copyright (C) 2008, 2014 Red Hat Inc.
# 
# This file is part of systemtap, and is free software.  You can
# redistribute it and/or modify it under the terms of the GNU General
# Public License (GPL); either version 2, or (at your option) any
# later version.

# This script monitors a remote system that is running the kprobes
# test.  If several consecutive 'ping's fail, the system is rebooted.
#
# This script takes as an argument a config filename, whose contents
# should look like the following:
#
#   config_opts['system_name'] = "SYSTEM_NAME"
#   config_opts['restart_cmds'] = [
#       'CMD1',
#       'CMD2',
#       ]
#
# As an example, here is a config file used when monitoring a kvm
# instance:
#
#   config_opts['system_name'] = "dhcp-148"
#   config_opts['restart_cmds'] = [
#       'sudo virsh destroy kvm-rawhide-64-1',
#       'sudo virsh start kvm-rawhide-64-1',
#       ]
#
# Here's a config file used then monitoring a beaker system
# (https://beaker-project.org/):
#
#   config_opts['system_name'] = "ibm-z10-38"
#   config_opts['restart_cmds'] = [
#       'bkr system-power --action reboot ibm-z10-38'
#       ]

import sys
import os
import time

if len(sys.argv) != 2:
    print >>sys.stderr, "Usage: %s config_file" % sys.argv[0]
    sys.exit(1)
cfg = sys.argv[1]

# Read in the config file
if not os.path.exists(cfg):
    print >>sys.stderr, ("Could not find required config file: %s" % cfg)
    sys.exit(1)

print "Reading config file %s..." % cfg
config_opts = dict()
execfile(cfg)
if not config_opts.has_key('system_name'):
    print >>sys.stderr, "Missing required config opt 'system_name'"
    sys.exit(1)
if not config_opts.has_key('restart_cmds'):
    print >>sys.stderr, "Missing required config opt 'restart_cmds'"
    sys.exit(1)

errors = 0
while 1:
    rc = os.system("ping -c 1 %s" % config_opts['system_name'])
    # If ping worked, system is still up and running.  Wait a minute
    # and try again.
    if os.WEXITSTATUS(rc) == 0:
        time.sleep(60)
        errors = 0

    # If the ping failed, increase the error count.  If we've got 3
    # consecutive errors, assume the machine has crashed and restart
    # it.
    else:
        errors += 1
        if errors < 3:
            time.sleep(30)
        else:
            print >>sys.stderr, "Restarting %s..." % config_opts['system_name']
            # Run each restart command

            for cmd in config_opts['restart_cmds']:
                print >>sys.stderr, "Running '%s'..." % cmd
                os.system(cmd)
            # Sleep for 5 minutes to give the system a chance to boot
            print >>sys.stderr, "Sleeping for 5 minutes..."
            time.sleep(5 * 60)
            errors = 0
            



#!/usr/bin/python

# Copyright (C) 2014 Red Hat Inc.
# 
# This file is part of systemtap, and is free software.  You can
# redistribute it and/or modify it under the terms of the GNU General
# Public License (GPL); either version 2, or (at your option) any
# later version.

import sys
import os
import os.path
import time
import select
import signal
from config_opts import config_opts

def run_module():
    logfile = 'data.log'
    os.system('rm -f ./data.log')

    # Insert the module
    print "Inserting module..."
    stap_cmd = [ "staprun", "-o", "data.log", "probe_module.ko" ]
    stap_pid = os.spawnvp(os.P_NOWAIT, stap_cmd[0], stap_cmd)
    print "stap pid is", stap_pid

    # Run the load generate commands (if any).  Note we ignore the
    # return values, since we don't really care if the commands
    # succeed or not.
    if config_opts.has_key('load_cmds'):
        print "Running commands..."

        # Redirect the output to 'run_module.log' by modifying
        # what stdout/stderr points to.
        sys.stdout.flush()
        old_stdout = os.dup(sys.stdout.fileno())
        sys.stderr.flush()
        old_stderr = os.dup(sys.stderr.fileno())
        fd = os.open('run_module.log', os.O_CREAT | os.O_WRONLY)
        os.dup2(fd, sys.stdout.fileno())
        os.dup2(fd, sys.stderr.fileno())

        # Run the commands.
        pid_list = list()
        for cmd in config_opts['load_cmds']:
            pid = os.spawnvp(os.P_NOWAIT, cmd[0], cmd)
            pid_list.append(pid)
        for pid in pid_list:
            (childpid, status) = os.waitpid(pid, 0)

        # Restore old value of stdout/stderr.
        os.close(fd)
        os.dup2(old_stdout, sys.stdout.fileno())
        os.dup2(old_stderr, sys.stderr.fileno())
    else:
        time.sleep(10)

    # Remove the module
    print "Removing module..."
    os.kill(stap_pid, signal.SIGINT)
    (childpid, rc) = os.waitpid(stap_pid, 0)
    #if os.WEXITSTATUS(rc) != 0:
    #    print >>sys.stderr, "Error: rmmod failed"
    #    return -1

    # Now we have to wait until everything is flushed to the logfile
    print "Looking for output..."
    rc = 0
    f = open(config_opts['probes_result'], 'w')
    attempts = 0
    start_pos = 0
    while 1:
        l = open(logfile, 'r')

        # Find the ending size of the logfile
        end_pos = os.path.getsize(logfile)

        # Try to wait until data is available
        while 1:
            try:
                input, output, exc = select.select([l.fileno()], [], [], 60)
                break
            except select.error, err:
                if err[0] != EINTR:
                    raise

        # Get the new stuff logged
        data = l.read(end_pos - start_pos + 1)
        if not data:
            # ignore EOF
            time.sleep(2)
            attempts += 1
            if attempts < 30:
                l.close()
                continue
            else:
                print >>sys.stderr, "Error: Couldn't find data"
                rc = -1
                break;

        # Write results data
        f.write(data)

        # See if we can find 'probe_module unloaded' in the data we
        # just read.
        if data.find('probe_module unloaded') == -1:
            start_pos = end_pos
        else:
            break

    l.close()
    f.close()
    return rc

def main():
    rc = run_module()
    sys.exit(rc)

if __name__ == "__main__":
    main()



#!/usr/bin/python

# Copyright (C) 2010 Red Hat Inc.
# 
# This file is part of systemtap, and is free software.  You can
# redistribute it and/or modify it under the terms of the GNU General
# Public License (GPL); either version 2, or (at your option) any
# later version.

import os
import pickle
import re
import sys
import time
from config_opts import config_opts
from gen_code import gen_module
from run_module import run_module

class BucketSet(object):
    def __init__(self, bucketA=list(), bucketA_result=0,
                 bucketB=list(), bucketB_result=0, buckets=list(),
                 passed=list(), failed=list(), untriggered=list(),
                 unregistered=list(), split=0): 
        # bucketA is the list of probes to test.
        self.bucketA = bucketA
        # bucketA_result tells us what state bucketA is in
        self.bucketA_result = bucketA_result
        # bucketB is the 2nd (optional) list of probes to test
        self.bucketB = bucketB
        # bucketB_result tells us what state bucketB is in
        self.bucketB_result = bucketB_result
        # buckets is the full list of probes to test (originally
        # filled in by reading config_opts['probes_all'])
        self.buckets = buckets
        # passed is the list of probe points that were sucessfully
        # registered and triggered (actually called)
        self.passed = passed
        # failed is the list of probe points that were sucessfully
        # registered but caused a crash.  The probe lists have been
        # bisected and singly eliminated.
        self.failed = failed
        # untriggered is the list of probe points that were sucessfully
        # registered and but never triggered (actually called)
        self.untriggered = untriggered
        # untriggered is the list of probe points that couldn't be
        # registered
        self.unregistered = unregistered
        # split tells us how to split a bucket.  0 means bisect
        # it, anything else is the number of times we've tried to do a
        # 1-by-1 split.
        self.split = split

    def bucketA_result_str(self):
        if self.bucketA_result == 0:
            return "untested"
        elif self.bucketA_result == 1:
            return "succeeded"
        elif self.bucketA_result == 2:
            return "failed"
        else:
            return "UNKNOWN"

    def bucketB_result_str(self):
        if self.bucketB_result == 0:
            return "untested"
        elif self.bucketB_result == 1:
            return "succeeded"
        elif self.bucketB_result == 2:
            return "failed"
        else:
            return "UNKNOWN"

probes = BucketSet()

# Install this script in config_opts['rclocal'] (typically
# /etc/rc.d/rc.local).  Returns true if the line wasn't already there
# (which means this is the 1st time the script has been run).
def register_script(install=True):
    rc = True
    regexp = re.compile("^.+/kprobes_test.py")
    f = open(config_opts['rclocal'], "r+")
    data = ""
    for line in f:
        if not regexp.match(line):
            data += line
        else:
            rc = False
    if install:
        data += "cd %s/ && ./kprobes_test.py &" % os.getcwd()
    f.seek(0, 0)
    f.write(data)
    f.truncate(f.tell())
    f.close()
    return rc

# Because it is very possible that we'll crash the system, we need to
# make sure our data files are written to the disk sucessfully before
# we run a module.  sync_disks() makes sure everything written is
# actually saved.
def sync_disks():
    os.system("sync; sync; sync")
    time.sleep(5)
    return

def read_probe_list():
    global probes

    # If the "pickled" file exists, read it in to recover our state
    if os.path.exists(config_opts['probes_db']):
        print "Reading state..."
        f = open(config_opts['probes_db'])
        p = pickle.Unpickler(f)
        probes = p.load()
        f.close()

    # if the 'probes_all' file exists, create the data from it
    elif os.path.exists(config_opts['probes_all']):
        # Read in the flat file
        f = open(config_opts['probes_all'])
        probe_lines = f.readlines()
        f.close()
        
        # Create the full bucket
        full_bucket = list()
        for line in probe_lines:
            full_bucket.append(line.rstrip())
        probes.buckets.append(full_bucket)

    # create the probes_all file?
    else:
        print >>sys.stderr, ("Could not find probes file")
        sys.exit(1)
    
def write_probe_list():
    global probes

    print "Writing state..."
    f = open(config_opts['probes_db'], 'w')
    p = pickle.Pickler(f)
    p.dump(probes)
    f.close()

def display_probe_list():
    global probes

    i = 0
    print "bucketA (%s) has %d entries" % \
        (probes.bucketA_result_str(), len(probes.bucketA))
    print "bucketB (%s) has %d entries" % \
        (probes.bucketB_result_str(), len(probes.bucketB))
    for bucket in probes.buckets:
        print "set %d has %d entries" % (i, len(bucket))
        i += 1
    print "passed set has %d entries" % len(probes.passed)
    total = 0
    for bucket in probes.failed:
        total += len(bucket)
    print "failed set has %d entries (in %d buckets)" % \
        (total, len(probes.failed))
    print "untriggered set has %d entries" % len(probes.untriggered)
    print "unregistered set has %d entries" % len(probes.unregistered)

def reset_buckets():
    global probes

    probes.bucketA = list()
    probes.bucketA_result = 0
    probes.bucketB = list()
    probes.bucketB_result = 0
    probes.split = 0

def grab_bucket():
    global probes

    reset_buckets()

    # Try to grab the 1st bucket from the list.
    if len(probes.buckets) > 0:
        bucket = probes.buckets[0]
        del probes.buckets[0]

        # if the bucket has more than 1000 probes, limit it to 1000
        if len(bucket) > 1000:
            probes.bucketA = bucket[0:1000]
            probes.bucketA_result = 0
            probes.bucketB_result = 0

            rest = bucket[1000:]
            probes.buckets.insert(0, rest)

        # otherwise just use the bucket
        else:
            probes.bucketA = bucket
        return True
    else:
        print "no buckets left"
        return False

def split_bucket(bucket):
    global probes

    if probes.split == 0:
        split = len(bucket) / 2
    else:
        split = 1
    bucketA = bucket[0:split]
    bucketB = bucket[split:]

    probes.bucketA = bucketA
    probes.bucketA_result = 0
    probes.bucketB = bucketB
    probes.bucketB_result = 0

def update_buckets(failed=True):
    global probes
    ret = True

    # if we don't have a current set, get one
    if len(probes.bucketA) == 0:
        ret = grab_bucket()

    # if bucketA is set, we've just finished up with it (or bucketB).
    elif len(probes.bucketA) > 0:
        if probes.bucketA_result == 0:
            if not failed:
                probes.bucketA_result = 1
            else:
                probes.bucketA_result = 2
            print "bucketA %s with %d probes..." % \
                (probes.bucketA_result_str(), len(probes.bucketA))
        else:
            if not failed:
                probes.bucketB_result = 1
            else:
                probes.bucketB_result = 2
            print "bucketB %s with %d probes..." % \
                (probes.bucketB_result_str(), len(probes.bucketB))

        # OK, we've got several cases here.
        # (1) Only bucketA was set.
        if len(probes.bucketB) == 0:
            print "case (1)..."
            # (1a) If bucketA passed, put it on the passed list and grab
            # the next bucket.
            if not failed:
                probes.passed.extend(probes.bucketA)
                probes.split = 0
                ret = grab_bucket()

            # (1b) If bucketA failed and is more than 1 probe, split it.
            elif len(probes.bucketA) > 1:
                split_bucket(probes.bucketA)

            # (1c) if bucketA failed and was only 1 probe, put it on
            # the failed list and grab the next bucket.
            else:
                probes.failed.append(probes.bucketA)
                ret = grab_bucket()

        # (2) Both bucketA and bucketB were set, but bucketB hasn't
        # been tested yet.
        elif probes.bucketB_result == 0:
            # Do nothing and let bucketB be tested.
            print "case (2)..."
            pass

        # (3) Both bucketA and bucketB have been tested.  Figure out
        # what to do next.
        else:
            print "case (3)..."

            # (3a) bucketA passed, bucketB failed.  Move bucketA to
            # the passed list and split bucketB.
            if probes.bucketA_result == 1 and probes.bucketB_result == 2:
                print "case (3a)..."
                probes.passed.extend(probes.bucketA)
                probes.split = 0

                # (3a1) If bucketB failed and is more than 1 probe,
                # split it.
                if len(probes.bucketB) > 1:
                    split_bucket(probes.bucketB)

                # (3a2) If bucketB failed and was only 1 probe, put it
                # on the failed list and grab the next bucket.
                else:
                    probes.failed.append(probes.bucketB)
                    ret = grab_bucket()

            # (3b) bucketA failed, bucketB passed.  Move bucketB to
            # the passed list and split bucketA.
            elif probes.bucketA_result == 2 and probes.bucketB_result == 1:
                print "case (3b)..."
                probes.passed.extend(probes.bucketB)
                probes.split = 0

                # (3b1) If bucketA failed and is more than 1 probe,
                # split it.
                if len(probes.bucketA) > 1:
                    split_bucket(probes.bucketA)

                # (3b2) If bucketA failed and was only 1 probe, put it
                # on the failed list and grab the next bucket.
                else:
                    probes.failed.append(probes.bucketA)
                    ret = grab_bucket()

            # (3c) Both buckets failed.
            elif probes.bucketA_result == 2 and probes.bucketB_result == 2:
                print "case (3c)..."
                # (3c1) bucketA and bucketB were both just 1 probe
                # each.  Put them both on the failed list and grab the
                # next bucket.
                if len(probes.bucketA) == 1 and len(probes.bucketB) == 1:
                    print "case (3c1)..."
                    probes.failed.append(probes.bucketA)
                    probes.failed.append(probes.bucketB)
                    ret = grab_bucket()

                # (3c2) bucketA was just 1 probe, but bucketB was more
                # than 1 probe
                elif len(probes.bucketA) == 1 and len(probes.bucketB) > 1:
                    print "case (3c2)..."
                    probes.failed.append(probes.bucketA)
                    split_bucket(probes.bucketB)
                # (3c3) bucketA was more than 1 probe, but bucketB was
                # just 1 probe
                elif len(probes.bucketA) > 1 and len(probes.bucketB) == 1:
                    print "case (3c3)..."
                    probes.failed.append(probes.bucketB)
                    split_bucket(probes.bucketA)
                # (3c4) Both buckets were more than 1 probe.  In this
                # case, split bucketA and put bucketB back on the main
                # list.  This will cause bucketB to get tested twice.
                else:
                    print "case (3c4)..."
                    probes.buckets.insert(0, probes.bucketB)
                    split_bucket(probes.bucketA)

            # (3d) Both buckets passed.  This sounds good, but this
            # means that together bucketA and bucketB failed, but
            # separately they passed.  So, the combination of the sets
            # is the problem.
            else:
                print "case (3d)..."
                # (3d1) If both buckets are just 1 probe, combine the
                # buckets into one and put it on the failed list.
                # Then grab the next bucket.
                if len(probes.bucketA) == 1 and len(probes.bucketB) == 1:
                    bucket = probes.bucketA + probes.bucketB
                    probes.failed.append(bucket)
                    ret = grab_bucket()

                # (3d2) Combine both buckets here, and try eliminating
                # the probes 1-by-1.  We'll go back to bisecting if we
                # can remove 1 probe from this list or the list is
                # indivisible (case 3d3a).
                elif probes.split == 0:
                    probes.split = 1
                    bucket = probes.bucketA + probes.bucketB
                    split_bucket(bucket)
                # (3d3) We're eliminating the probes 1-by-1 and the
                # last attempt failed.  Reverse the order and try
                # again.
                else:
                    probes.split += 1
                    print "case (3d3): bucketA(%d) bucketB(%d)" % (len(probes.bucketA), len(probes.bucketB))
                    bucket = probes.bucketB + probes.bucketA
                    print "case (3d3): combined bucket(%d)" % (len(bucket))
                    if len(bucket) > 500:
                        print >>sys.stderr, "Error: bucket grew?"
                        return -1
                    
                    # (3d3a) If we've tried every probe singly, and
                    # they all still worked, the combination is still
                    # the problem.  So, we're done trying 1-by-1
                    # elimination.
                    if probes.split > len(bucket):
                        probes.split = 0
                        probes.failed.append(bucket)
                        ret = grab_bucket()
                    # (3d3b) Keep trying 1-by-1 splits.
                    else:
                        split_bucket(bucket)

    write_probe_list()
    return ret

def save_bucket():
    global probes

    f = open(config_opts['probes_current'], 'w')
    if len(probes.bucketA) > 0 and probes.bucketA_result == 0:
        f.write("\n".join(probes.bucketA))
    elif len(probes.bucketB) > 0 and probes.bucketB_result == 0:
        f.write("\n".join(probes.bucketB))
    else:
        print "Error: no bucket to write?"
    f.close()

def parse_module_output():
    global probes

    # Parse the output file, looking for probe points
    pp_re = re.compile(": (-?\d+) (\S+)$")
    f = open(config_opts['probes_result'], 'r')
    pp = dict()
    line = f.readline()
    while line:
        match = pp_re.search(line)
        if match:
            pp[match.group(2)] =  int(match.group(1))
        line = f.readline()
    f.close()

    if len(pp.keys()) == 0:
        print >>sys.stderr, "No data found?"
        return 1

    # We're done with the result file
    os.unlink(config_opts['probes_result'])

    # Parse the list of probe points.  Since the result fields haven't
    # been updated yet, pick the 1st bucket that has a status of 0
    # (untested).
    if probes.bucketA_result == 0:
        bucket = probes.bucketA
    else:
        bucket = probes.bucketB
        
    new_bucket = list()
    for probe in bucket:
        if pp.has_key(probe):
            # > 0 == passed (registered and triggered)
            if pp[probe] > 0:
                new_bucket.append(probe)
            # 0 == untriggered
            elif pp[probe] == 0:
                probes.untriggered.append(probe)
            # -1 == unregistered
            elif pp[probe] == -1:
                probes.unregistered.append(probe)
            else:
                print >>sys.stderr, "failed probe %s?" % probe
        else:
            print >>sys.stderr, "Couldn't find %s?" % probe

    # OK, we've gone through and removed all the
    # untriggered/unregistered probes from bucket.  Update the
    # proper bucket.
    if probes.bucketA_result == 0:
        # Oops, all the probes were unregistered/untriggered.
        if len(new_bucket) == 0:
            # If we've got a bucketB, we'll think it just succeeded.
            # So, put it back on the bucket list so it will get
            # correctly processed.
            if len(probes.bucketB):
                print "Re-inserting bucketB..."
                probes.buckets.insert(0, probes.bucketB)

            # Reset everything
            reset_buckets()
        else:
            probes.bucketA = new_bucket
    else:
        # If bucketB ended up with 0 entries, we need to handle
        # bucketA.
        if len(new_bucket) == 0:
            # bucketA suceeded and bucketB ended up with 0 entries.
            # Put bucketA on the 'passed' list.
            if probes.bucketA_result == 1:
                probes.passed.extend(probes.bucketA)
            # bucketA failed and bucketB ended up with 0 entries.
            # Put bucketA on the bucket list to get processed again.
            # This will cause it to get tested twice, but it is the
            # easiest way out.
            else:
                print "Re-inserting bucketA..."
                probes.buckets.insert(0, probes.bucketA)

            # Reset everything.
            reset_buckets()
        else:
            probes.bucketB = new_bucket

    return 0

def run_tests():
    status = True
    failed = True
    while status:
        status = update_buckets(failed)
        display_probe_list()
        if not status:
            break

        # Generate the module.
        save_bucket()
        rc = gen_module()
        if rc != 0:
            sys.exit(rc)

        # Run the module.
        sync_disks()
        rc = run_module()
        if rc != 0:
            sys.exit(rc)

        # Parse the module output.
        rc = parse_module_output()
        if rc != 0:
            sys.exit(rc)

        # If we're here, the current module was loaded and unloaded
        # successfully.
        failed = False

def dump_output():
    global probes
    
    f = open(config_opts['probes_passed'], 'w')
    if len(probes.passed) > 0:
        f.write("\n".join(probes.passed))
    f.close()

    # probes.failed is a list of lists.
    f = open(config_opts['probes_failed'], 'w')
    for bucket in probes.failed:
        f.write("\n".join(bucket))
        f.write("\n#\n")
    f.close()

    f = open(config_opts['probes_untriggered'], 'w')
    if len(probes.untriggered) > 0:
        f.write("\n".join(probes.untriggered))
    f.close()

    f = open(config_opts['probes_unregistered'], 'w')
    if len(probes.unregistered) > 0:
        f.write("\n".join(probes.unregistered))
    f.close()

# Make sure we're running as root.
if os.getuid() != 0:
    print >>sys.stderr, "Error: this script must be run by root"
    sys.exit(1)

# Register this script.  If this is the 1st time we've been run, start
# from scratch by removing old state files.
if register_script():
    print >>sys.stderr, "Removing old state files..."
    if os.path.exists(config_opts['log_file']):
        os.unlink(config_opts['log_file'])
    if os.path.exists(config_opts['probes_db']):
        os.unlink(config_opts['probes_db'])

# Redirect stdout and stderr
sys.stdout.flush()
sys.stderr.flush()
so = open(config_opts['log_file'], 'a+', 0)           # no buffering
os.dup2(so.fileno(), sys.stdout.fileno())
os.dup2(so.fileno(), sys.stderr.fileno())
    
# Go!
read_probe_list()
run_tests()

# Finish up.
dump_output()
register_script(False)
print >>sys.stderr, "Finished."



#!/usr/bin/python
# Copyright (C) 2014 Red Hat, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.

import sys

aglobal9 = 9

def celsius_to_farenheit(celsius):
    atuple = "a", "b", "c"
    alist = [1, 2, 3]
    aset = {1, 2, 3}
    adict = { 1 : "a", 2 : "b", 3 : "c" }

    nine = aglobal9
    five = 5
    thirty_two = 32
    i = 1
    return str((nine * celsius) / five + thirty_two)

def main():
    if (len(sys.argv) < 2):
        print ("Usage: " + sys.argv[0] + " Temp")
        return 1

    celsius = int(sys.argv[1])
    print (str(celsius) + " Celsius " + " is " + celsius_to_farenheit(celsius) + " Farenheit")
    return 0

if __name__ == "__main__":
    sys.exit(main())



#!/usr/bin/python

# Copyright (C) 2014 Red Hat Inc.
# 
# This file is part of systemtap, and is free software.  You can
# redistribute it and/or modify it under the terms of the GNU General
# Public License (GPL); either version 2, or (at your option) any
# later version.

import os
import sys
from config_opts import config_opts

def gen_files(dir, subset):
    f = open('%s/probes.stp' % dir, 'w')

    # Output the script
    i = 0
    while i < len(subset):
        print >>f, ("global called%d" % (i))
        i += 1
    print >>f, "probe begin(-1)"
    print >>f, "{"
    i = 0
    while i < len(subset):
        print >>f, ("\tcalled%d <<< 0" % (i))
        i += 1
    print >>f, "}"
    i = 0
    while i < len(subset):
        print >>f, ("probe kernel.function(\"%s\").call { called%d <<< 1 }" % (subset[i], i))
        i += 1
    print >>f, "probe end"
    print >>f, "{"
    i = 0
    while i < len(subset):
        print >>f, ("\tprintf(\"%%d %s\\n\", @sum(called%d))" % (subset[i], i))
        i += 1
    print >>f, "\tprintf(\"probe_module unloaded\\n\")"
    print >>f, "}"
    f.close()

def gen_module():
    f = open(config_opts['probes_current'])
    probes = f.readlines()
    f.close()
    if len(probes) == 0:
        print >>sys.stderr, ("Error: no probe points in %s"
                             % config_opts['probes_current'])
        return -1

    # Cleanup each probe by stripping whitespace
    i = 0
    while i < len(probes):
        probes[i] = probes[i].rstrip()
        i += 1

    # Generate necessary files
    gen_files(os.getcwd(), probes)

    # Try to build the module
    os.system('rm -f ./probe_module.ko')

    cmd = "stap -g --suppress-handler-errors --suppress-time-limits -vp4 -m probe_module probes.stp >build.log 2>&1"
    print "Running", cmd
    rc = os.system(cmd)
    if os.WEXITSTATUS(rc) != 0:
        print >>sys.stderr, "Error: stap failed, see build.log for details"
        return -1
    return 0

def main():
    rc = gen_module()
    sys.exit(rc)

if __name__ == "__main__":
    main()



# Copyright (C) 2008, 2010, 2014 Red Hat Inc.
# 
# This file is part of systemtap, and is free software.  You can
# redistribute it and/or modify it under the terms of the GNU General
# Public License (GPL); either version 2, or (at your option) any
# later version.

import os
import sys

# Here we set up the default config options.  These can be overridden
# by the config file.
config_opts = dict()

#
# Various file names
#

# PROBES_ALL is the file that contains the full list of kernel
# functions to test.  If it doesn't exist, it can be created by
# readelf.py with a list of all kernel functions.
config_opts['probes_all'] = 'probes.all'

# PROBES_UNREGISTERED is a file that contains the list of kernel
# functions where a probe couldn't be registered.
config_opts['probes_unregistered'] = 'probes.unregistered'

# PROBES_UNTRIGGERED is a file that contains the list of kernel
# functions where a probe could be registered, but the kernel
# function wasn't actually called.  So, we don't really know whether
# these functions passed or failed.
config_opts['probes_untriggered'] = 'probes.untriggered'

# PROBES_PASSED is a file that contains the list of kernel functions
# where a probe was registered and the kerenel function was called
# with no crash.
config_opts['probes_passed'] = 'probes.passed'

# PROBES_FAILED is a file that contains the list of kernel functions
# where a probe was registered and the kernel function was called
# that caused a crash.
config_opts['probes_failed'] = 'probes.failed'

# PROBES_CURRENT is a file that contains the list of kernel functions
# we're about to test.
config_opts['probes_current'] = 'probes.current'

# PROBES_RESULT is a file that contains the results of the current
# test.
config_opts['probes_result'] = 'probe.out'

# PROBES_DB is a file that contains the current state (in python
# 'pickled' format).
config_opts['probes_db'] = 'probes.db'

# LOG_FILE is the file that contains the testing log output.
config_opts['log_file'] = 'kprobes_test.log'

# RCLOCAL is the file we need to edit to get the test to automatically
# run after a reboot.
config_opts['rclocal'] = '/etc/rc.d/rc.local'

# Read in the config file
print "Reading config file..."
cfg = os.path.join(os.getcwd(), 'default.cfg')
if os.path.exists(cfg):
    execfile(cfg)
else:
    print >>sys.stderr, ("Could not find required config file: %s" % cfg)
    sys.exit(1)



#!/usr/bin/python

# Copyright (C) 2008 Red Hat Inc.
# 
# This file is part of systemtap, and is free software.  You can
# redistribute it and/or modify it under the terms of the GNU General
# Public License (GPL); either version 2, or (at your option) any
# later version.

import re
import sys
import os
#import pickle
import subprocess
from config_opts import config_opts

# Read the output of eu-readelf on vmlinux
(sysname, nodename, release, version, machine) = os.uname()
cmd = "eu-readelf --symbols /usr/lib/debug/lib/modules/%s/vmlinux" % release
print "Running", cmd
p = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE)
lines = p.stdout.readlines()
p.wait()
if p.returncode != 0:
    print >>sys.stderr, "Error: eu-readelf failed."
    sys.exit(p.returncode)

# Parse the output
kprobes_text_start = 0
kprobes_text_end = 0
syms = dict()
func_re = re.compile("^\s*\d+:\s+([0-9a-f]+)\s+\d+\s+FUNC\s+\S+\s+\S+\s+\d+\s+(\S+)$")
notype_re = re.compile("^\s*\d+:\s+([0-9a-f]+)\s+\d+\s+NOTYPE\s+\S+\s+\S+\s+\d+\s+(\S+)$")
for line in lines:
    match = notype_re.match(line)
    if match:
        addr = match.group(1)
        name = match.group(2)
        if name == "__kprobes_text_start":
            kprobes_text_start = long(addr, 16)
        elif name == "__kprobes_text_end":
            kprobes_text_end = long(addr, 16)
        continue

    match = func_re.match(line)
    if match:
        addr = match.group(1)
        name = match.group(2)
        syms[name] = long(addr, 16)

# Now we've parsed everything.  Now we need to go back and remove all
# symbols between '__kprobes_text_start' and '__kprobes_text_end',
# since they are already protected from kprobes.  We couldn't do this
# in the loop above, since we might encounter symbols that need to be
# removed before we found the start/end of the kprobes section.
if kprobes_text_start == 0 or kprobes_text_end == 0:
    print "Error - didn't find kprobes_test_start(%d) or kprobes_text_end(%d)" \
        % (kprobes_text_start, kprobes_text_end)
    sys.exit(1)

for name in syms.keys():
    if syms[name] >= kprobes_text_start and syms[name] < kprobes_text_end:
        print "deleting", name
        del syms[name]

## Save data
#f = open('%s.syms' % (release), 'w')
#p = pickle.Pickler(f)
#p.dump(syms)
#f.close()

# Write the data out in text format
f = open(config_opts['probes_all'], 'w')
for name in syms.keys():
    print >>f, name
f.close()



#!/usr/bin/python

# Copyright (C) 2008, 2010 Red Hat Inc.
# 
# This file is part of systemtap, and is free software.  You can
# redistribute it and/or modify it under the terms of the GNU General
# Public License (GPL); either version 2, or (at your option) any
# later version.

import os
import sys
from config_opts import config_opts

def gen_files(dir, subset):
    f = open('%s/kprobe_defs.h' % dir, 'w')

    # Output the array of kp_data structs
    print >>f, "static struct kp_data kp_data[] = {"
    i = 0
    while i < len(subset):
        print >>f, ("\t{ .kp={ .symbol_name=\"%s\", .pre_handler=&handler_pre, }, .use_count=ATOMIC_INIT(0) },"
                    % (subset[i]))
        i += 1
    print >>f, "};"
    print >>f
    f.close()

    # Generate the Makefile
    f = open('Makefile', 'w')
    print >>f, """
EXTRA_CFLAGS :=
EXTRA_CFLAGS += -freorder-blocks
EXTRA_CFLAGS += -Wno-unused -Werror
obj-m := kprobe_module.o"""
    f.close()

def run_make_cmd(cmd):
    # Before running make, fix up the environment a bit.  Clean out a
    # few variables that /lib/modules/${KVER}/build/Makefile uses.
    os.unsetenv("ARCH")
    os.unsetenv("KBUILD_EXTMOD")
    os.unsetenv("CROSS_COMPILE")
    os.unsetenv("KBUILD_IMAGE")
    os.unsetenv("KCONFIG_CONFIG")
    os.unsetenv("INSTALL_PATH");

    print "Running", cmd
    return os.system(cmd)

def gen_module():
    f = open(config_opts['probes_current'])
    probes = f.readlines()
    f.close()
    if len(probes) == 0:
        print >>sys.stderr, ("Error: no probe points in %s"
                             % config_opts['probes_current'])
        return -1

    # Cleanup each probe by stripping whitespace
    i = 0
    while i < len(probes):
        probes[i] = probes[i].rstrip()
        i += 1

    # Generate necessary files
    gen_files(os.getcwd(), probes)

    # Try to build the module - add "V=1" at the end for more verbosity
    os.system('rm -f ./kprobe_module.ko')
    (sysname, nodename, release, version, machine) = os.uname()
    cmd = ("make -C \"/lib/modules/%s/build\" M=\"%s\" modules CONFIG_MODULE_SIG=n >build.log 2>&1"
           % (release, os.getcwd()))
    rc = run_make_cmd(cmd)
    if os.WEXITSTATUS(rc) != 0:
        print >>sys.stderr, "Error: Make failed, see build.log for details"
        return -1
    return 0

def main():
    rc = gen_module()
    sys.exit(rc)

if __name__ == "__main__":
    main()
